#include                <unistd.h>
#include                <string.h>
#include                <stdlib.h>
#include                <dlfcn.h>
#include <pthread.h>
#include <stdio.h>
#include		"tap.h"
#include		"libFireBird.h"
#include    "TMSCommander.h" //v1.1

#define PROGRAM_NAME	"AutoReboot"
#define VERSION			"V1.1"
#define D(x)			;

TAP_ID				(0x8E0Dffff);
TAP_PROGRAM_NAME	(PROGRAM_NAME" "VERSION);
TAP_AUTHOR_NAME		("srupprecht & xyzzyx");
TAP_DESCRIPTION		("Reboots your freezed toppy");
TAP_ETCINFO			(__DATE__);

static pthread_t thread;
static dword  _currenttick = 0;
static int    _done = 0, _writelog = 1;
static word   _year;
static char   _month, _day, _hrs, _min, _sec;

void *watchdog(void *arg);
static dword TMSCommander_handler(dword param1); //v1.1
dword TAP_EventHandler(word event, dword param1, dword param2);
int TAP_Main(void);

void *watchdog(void *arg)
{
	int   timeout = (int) arg, counter = timeout;
	dword lasttick = _currenttick;
	
	D(TAP_Print("child is awake, timeout is %d secs.\n", timeout));
	
	do
	{
		sleep(1);
		
		D(TAP_Print("ticking\n"));

		if (lasttick != _currenttick)
		{
			lasttick = _currenttick;
			counter  = timeout;
		}
		else if (--counter <= 0)
		{
			D(TAP_Print("reboot!\n"));

			if (_writelog)
			{
				FILE *fp;

				if ((fp = fopen("/mnt/hd/ProgramFiles/AutoReboot.log", "a")))
				{
					fprintf(fp, "Toppy freezed on %02d.%02d.%04d at "\
								"%02d:%02d:%02d o'clock.\n",
							_day, _month, _year, _hrs, _min, _sec);
					fclose(fp);
				}
			}
			
			system("reboot");
			break;
		}
	}
	while (!_done);

	D(TAP_Print("received end signal!\n"));
	
	pthread_exit(NULL);
}

// TMSCommander_handler //v1.1
static dword TMSCommander_handler(dword param1)
{
	switch (param1)
	{
		case TMSCMDR_Capabilities:
			return (dword)(TMSCMDR_CanBeStopped);

		case TMSCMDR_Stop:
		//Exit your TAP
			D(TAP_Print("EVT_STOP, stopping thread...\n"));
			_done = 1;
			pthread_join(thread, 0);
			TAP_Exit(); //v1.1
			return TMSCMDR_OK;

		case TMSCMDR_Exiting:
		//TMSCommander is exiting - do you want to do anything?
			return TMSCMDR_OK;

		default:
		//unknown TMSCommander function
			return TMSCMDR_UnknownFunction;
	}
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
  (void) param2;

  if (event == EVT_TMSCommander) //v1.1
		return TMSCommander_handler(param1); //v1.1

  if (event == EVT_IDLE)
  {
	static dword nexttime = 0;
	dword timenow = TAP_GetTick();

	if (timenow >= nexttime)
	{
		if (_writelog)
		{
			word   mjd;
			char   wday;

			TAP_GetTime(&mjd, &_hrs, &_min, &_sec);
			TAP_ExtractMjd(mjd, &_year, &_month, &_day, &wday);
		}

		_currenttick = timenow;
		nexttime     = timenow + 100;
	}
  }
  else if (event == EVT_STOP)
  {
		D(TAP_Print("EVT_STOP, stopping thread...\n"));
		_done = 1;
		pthread_join(thread, 0);
		TAP_Exit(); //v1.1
  }

  return param1;
}

int TAP_Main(void)
{
	int timeout = 30;

	if (INIOpenFile(PROGRAM_NAME".ini", PROGRAM_NAME) != INILOCATION_NotFound)
	{
		timeout = INIGetInt("TimeOut",30,10,3600);
		_writelog  = INIGetInt("WriteLog",1,0,1);
	}
	
	INICloseFile();
		
	D(TAP_Print("starting thread\n"));

	if (pthread_create(&thread, NULL, watchdog, (void *)timeout))
	{
		D(TAP_Print("pthread_create() failed!\n"));
		return 0;
	}
	
	return 1;
}
